library ppp.debug;
import 'petitparser.dart';
import 'reflection.dart';
typedef void OutputHandler(Object object);
String _repeat(int count, String value) {
  var result = new StringBuffer();
  for (var i = 0; i < count; i++) {
    result.write(value);
  }
  return result.toString();
}
typedef Result ContinuationHandler(Result continuation(Context context), Context context);
class ContinuationParser extends DelegateParser {
  final ContinuationHandler handler;
  ContinuationParser(parser, this.handler) : super(parser);
  @override Result parseOn(Context context) {
    return handler((result) => super.parseOn(result), context);
  }
  @override Parser copy() => new ContinuationParser(children[0], handler);
}
Parser profile(Parser root, [OutputHandler output = print]) {
  var count = new Map();
  var watch = new Map();
  var parsers = new List();
  return new ContinuationParser(transformParser(root, (parser) {
    parsers.add(parser);
    return new ContinuationParser(parser, (continuation, context) {
      count[parser]++;
      watch[parser].start();
      var result = continuation(context);
      watch[parser].stop();
      return result;
    });
  }), (continuation, context) {
    parsers.forEach((parser) {
      count[parser] = 0;
      watch[parser] = new Stopwatch();
    });
    var result = continuation(context);
    parsers.forEach((parser) {
      output('${count[parser]}\t' '${watch[parser].elapsedMicroseconds}\t' '${parser}');
    });
    return result;
  });
}
Parser progress(Parser parser, [OutputHandler output = print]) {
  return transformParser(parser, (each) {
    return new ContinuationParser(each, (continuation, context) {
      output('${_repeat(1 + context.position, '*')} $each');
      return continuation(context);
    });
  });
}
Parser trace(Parser parser, [OutputHandler output = print]) {
  var level = 0;
  return transformParser(parser, (each) {
    return new ContinuationParser(each, (continuation, context) {
      output('${_repeat(level, '  ')}${each}');
      level++;
      var result = continuation(context);
      level--;
      output('${_repeat(level, '  ')}${result}');
      return result;
    });
  });
}
