library ppp.json;
import 'dart:collection';
import 'petitparser.dart';
class JsonGrammar extends GrammarParser {
  JsonGrammar() : super(new JsonGrammarDefinition());
}
class JsonGrammarDefinition extends GrammarDefinition {
  start() => ref(value).end();
  token(p) => p.flatten().trim();
  array() => ref(token, char('[')) & ref(elements).optional() & ref(token, char(']'));
  elements() => ref(value).separatedBy(ref(token, char(',')), includeSeparators: false);
  members() => ref(pair).separatedBy(ref(token, char(',')), includeSeparators: false);
  object() => ref(token, char('{')) & ref(members).optional() & ref(token, char('}'));
  pair() => ref(stringToken) & ref(token, char(':')) & ref(value);
  value() => ref(stringToken) | ref(numberToken) | ref(object) | ref(array) | ref(trueToken) | ref(falseToken) | ref(nullToken);
  trueToken() => ref(token, string('true'));
  falseToken() => ref(token, string('false'));
  nullToken() => ref(token, string('null'));
  stringToken() => ref(token, ref(stringPrimitive));
  numberToken() => ref(token, ref(numberPrimitive));
  characterPrimitive() => ref(characterNormal) | ref(characterEscape) | ref(characterOctal);
  characterNormal() => pattern('^"\\');
  characterEscape() => char('\\') & pattern(new List.from(JSON_ESCAPE_CHARS.keys).join());
  characterOctal() => string('\\u').seq(pattern("0-9A-Fa-f").times(4).flatten());
  numberPrimitive() => char('-').optional() & char('0').or(digit().plus()) & char('.').seq(digit().plus()).optional() & pattern('eE').seq(pattern('-+').optional()).seq(digit().plus()).optional();
  stringPrimitive() => char('"') & ref(characterPrimitive).star() & char('"');
}
const JSON_ESCAPE_CHARS = const {
  '\\': '\\',
  '/': '/',
  '"': '"',
  'b': '\b',
  'f': '\f',
  'n': '\n',
  'r': '\r',
  't': '\t'
};
class JsonParser extends GrammarParser {
  JsonParser() : super(new JsonParserDefinition());
}
class JsonParserDefinition extends JsonGrammarDefinition {
  array() => super.array().map((each) => each[1] != null ? each[1] : new List());
  object() => super.object().map((each) {
    var result = new LinkedHashMap();
    if (each[1] != null) {
      for (var element in each[1]) {
        result[element[0]] = element[2];
      }
    }
    return result;
  });
  trueToken() => super.trueToken().map((each) => true);
  falseToken() => super.falseToken().map((each) => false);
  nullToken() => super.nullToken().map((each) => null);
  stringToken() => ref(stringPrimitive).trim();
  numberToken() => super.numberToken().map((each) {
    var floating = double.parse(each);
    var integral = floating.toInt();
    if (floating == integral && each.indexOf('.') == -1) {
      return integral;
    } else {
      return floating;
    }
  });
  stringPrimitive() => super.stringPrimitive().map((each) => each[1].join());
  characterEscape() => super.characterEscape().map((each) => JSON_ESCAPE_CHARS[each[1]]);
  characterOctal() => super.characterOctal().map((each) {
    throw new UnsupportedError('Octal characters not supported yet');
  });
}
