library ppp.reflection;
import 'dart:collection';
import 'petitparser.dart';
Iterable<Parser> allParser(Parser root) => new _ParserIterable(root);
class _ParserIterable extends IterableBase<Parser> {
  final Parser root;
  _ParserIterable(this.root);
  @override Iterator<Parser> get iterator => new _ParserIterator([root]);
}
class _ParserIterator implements Iterator<Parser> {
  final List<Parser> todo;
  final Set<Parser> seen;
  _ParserIterator(Iterable<Parser> roots)
      : todo = new List.from(roots),
        seen = new Set.from(roots);
  @override Parser current;
  @override bool moveNext() {
    if (todo.isEmpty) {
      current = null;
      return false;
    }
    current = todo.removeLast();
    for (var parser in current.children) {
      if (!seen.contains(parser)) {
        todo.add(parser);
        seen.add(parser);
      }
    }
    return true;
  }
}
Parser removeSettables(Parser parser) {
  return transformParser(parser, (each) {
    while (each is SettableParser) {
      each = each.children.first;
    }
    return each;
  });
}
Parser removeDuplicates(Parser parser) {
  var uniques = new Set();
  return transformParser(parser, (source) {
    var target = uniques.firstWhere((each) {
      return source != each && source.isEqualTo(each);
    }, orElse: () => null);
    if (target == null) {
      uniques.add(source);
      return source;
    } else {
      return target;
    }
  });
}
typedef Parser TransformationHandler(Parser parser);
Parser transformParser(Parser parser, TransformationHandler handler) {
  var mapping = new Map.identity();
  for (var each in allParser(parser)) {
    mapping[each] = handler(each.copy());
  }
  var seen = new Set.from(mapping.values);
  var todo = new List.from(mapping.values);
  while (todo.isNotEmpty) {
    var parent = todo.removeLast();
    for (var child in parent.children) {
      if (mapping.containsKey(child)) {
        parent.replace(child, mapping[child]);
      } else if (!seen.contains(child)) {
        seen.add(child);
        todo.add(child);
      }
    }
  }
  return mapping[parser];
}
